import setuptools
from skbuild import setup
import sys


try:
    from skbuild.command.bdist_wheel import bdist_wheel
    class CustomBdistWheel(bdist_wheel):
        def get_tag(self):
            rv = bdist_wheel.get_tag(self)
            return (self.python_tag, 'none', *rv[2:])
except ImportError:
    CustomBdistWheel = None


if sys.platform == "win32":
    # download libusb zip, and force use of msvc
    cmake_args = ['-DCMAKE_C_COMPILER=cl.exe', '-DCMAKE_CXX_COMPILER=cl.exe']
    is_64bit = sys.maxsize > (2 ** 32)
    if is_64bit:
        cmake_install_dir = 'asphodel/lib64'
    else:
        cmake_install_dir = 'asphodel/lib32'
else:
    if sys.platform == "darwin":
        cmake_args = ['-DCMAKE_INSTALL_RPATH=@loader_path']
    else:
        cmake_args = ['-DCMAKE_INSTALL_RPATH=$ORIGIN',

                      # limit to 2 parallel jobs for building on raspberry pi
                      '-DCMAKE_JOB_POOL_COMPILE:STRING=compile',
                      '-DCMAKE_JOB_POOL_LINK:STRING=link',
                      '-DCMAKE_JOB_POOLS:STRING=compile=2;link=2']
    cmake_install_dir = 'asphodel/lib'


def no_local_develop_scheme(version):
    if version.branch == "develop" and not version.dirty:
        return ""
    else:
        from setuptools_scm.version import get_local_node_and_date
        return get_local_node_and_date(version)


with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name="asphodel",
    use_scm_version={'write_to': 'asphodel/version.py',
                     'local_scheme': no_local_develop_scheme},
    setup_requires=['setuptools_scm'],
    author="Suprock Technologies, LLC",
    author_email="inquiries@suprocktech.com",
    description="Python wrapper for the Asphodel C library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://bitbucket.org/suprocktech/asphodel_py",
    packages=setuptools.find_packages(),
    keywords="asphodel suprock",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: ISC License (ISCL)",
        "Operating System :: OS Independent",
        "Topic :: System :: Hardware",
    ],
    python_requires=">=3.4",
    install_requires=[
        "numpy",
    ],
    entry_points={
        'console_scripts': [
            'nvmeditor = asphodel.nvmeditor:main',
        ],
    },
    zip_safe=False,
    cmake_install_dir=cmake_install_dir,
    cmake_args=cmake_args,
    include_package_data=False,
    exclude_package_data={'asphodel': ['lib*/*.lib', 'lib*/include/*']},
    cmdclass={'bdist_wheel': CustomBdistWheel},
)



#!/usr/bin/env python3
import argparse
import os.path
import shutil
import subprocess
import sys
import zipfile


def report_bitbucket_status(auth, commit, status):
    import requests
    endpoint = ("https://api.bitbucket.org/2.0/repositories/suprocktech/"
                "asphodel/commit/{}/statuses/build").format(commit)
    requests.post(endpoint, auth=auth, data=status)


def run_cmake(args):
    path = os.environ["PATH"]
    path += os.pathsep + r"C:\Program Files\CMake\bin"
    path += os.pathsep + r"C:\Program Files (x86)\CMake\bin"
    cmake = shutil.which("cmake", path=path)
    cmake_args = [cmake]
    cmake_args.extend(args)
    subprocess.check_call(cmake_args)


def create_zip(zip_name, directory):
    with zipfile.ZipFile(zip_name, 'w', zipfile.ZIP_DEFLATED) as zip:
        for root, dirs, files in os.walk(directory):
            for file in files:
                fullpath = os.path.join(root, file)
                relpath = os.path.relpath(fullpath, start=directory)
                zip.write(fullpath, arcname=relpath)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-b", dest="build_dir", required=True,
                        help="build directory")
    parser.add_argument("-G", dest="generator", required=True,
                        help="CMake generator")
    parser.add_argument("-z", dest="zip", help="Zip file name")
    parser.add_argument("-t", dest="type", default="RelWithDebInfo",
                        help='CMake build type')
    parser.add_argument("-c", dest="commit", metavar="HASH",
                        help='commit id to send to bitbucket')
    parser.add_argument("-k", dest="status_key", metavar="KEY",
                        help='key sent to bitbucket (e.g. "win32")')
    parser.add_argument("-n", dest="status_name", metavar="NAME",
                        help='name sent to bitbucket (e.g. "Win32 Binaries")')
    parser.add_argument("-u", dest="status_url", metavar="URL",
                        help='INPROGRESS and FAILED url passed to bitbucket')
    parser.add_argument("-s", dest="s3_bucket", metavar="S3_BUCKET",
                        help='S3 bucket for uploading',
                        default='suprock-public-software')
    parser.add_argument("-p", dest="s3_prefix", metavar="S3_PREFIX",
                        help='S3 prefix when uploading', default='')
    parser.add_argument("cmake_args", nargs="*",
                        help="Extra arguments passed to CMake")
    args = parser.parse_args()
    
    bb_args = ['commit', 'status_key', 'status_name', 'status_url']
    bb_args_defined = [getattr(args, k) is not None for k in bb_args]
    if any(bb_args_defined):
        if not all(bb_args_defined):
            parser.error('all bitbucket arguments must be given together')

        if 'BB_AUTH_STRING' not in os.environ:
            raise Exception("BB_AUTH_STRING environment var must be defined")
        auth = tuple(os.environ['BB_AUTH_STRING'].split(":", 1))
        status = {"state": "INPROGRESS",
                  "key": args.status_key,
                  "name": args.status_name,
                  "url": args.status_url}
        report_status = True
    else:
        report_status = False

    if report_status:
        report_bitbucket_status(auth, args.commit, status)

    script_dir = os.path.dirname(os.path.abspath(__file__))
    root_dir = os.path.abspath(os.path.join(script_dir, ".."))
    build_dir = os.path.join(root_dir, args.build_dir)
    output_dir = os.path.join(build_dir, "output")

    # make the build dir
    os.makedirs(build_dir, exist_ok=True)

    # cd to the build dir
    os.chdir(build_dir)

    try:
        # configure
        configure_args = ["-G", args.generator,
                          "-DCMAKE_BUILD_TYPE=" + args.type,
                          "-DCMAKE_INSTALL_PREFIX=" + output_dir]
        configure_args.extend(sys.argv[1:])
        configure_args.append(root_dir)
        run_cmake(configure_args)

        # build
        run_cmake(["--build", ".", "--config", args.type,
                   "--target", "install"])

        # create zip file
        if args.zip:
            create_zip(args.zip, output_dir)

            if report_status:
                # try uploading to S3
                import boto3
                s3 = boto3.client('s3')
                key = "{}{}".format(args.s3_prefix, args.zip)
                s3.upload_file(args.zip, args.s3_bucket, key)
                zip_url = 'https://{}.s3.amazonaws.com/{}'
                status['url'] = zip_url.format(args.s3_bucket, key)

        if report_status:
            status['state'] = "SUCCESSFUL"
            report_bitbucket_status(auth, args.commit, status)
    except:
        if report_status:
            status['state'] = "FAILED"
            report_bitbucket_status(auth, args.commit, status)
        raise


if __name__ == "__main__":
    main()
